@@ -5,7 +5,10 @@ import os
 import random
 import re
 import socket
-import cStringIO as StringIO
+try:
+    import cStringIO as StringIO
+except ImportError:
+    import io as StringIO
 import sys
 try:
     import ssl
@@ -31,7 +34,10 @@ _untested = []
 def untested(meth):
     """Decorator to mark a method as untested"""
     meth.untested = True
-    _untested.append(meth.func_name)
+    if hasattr(meth, 'func_name'):
+        _untested.append(meth.func_name)
+    else:
+        _untested.append(meth.__name__)
     return meth
 
 class IloError(Exception):
@@ -167,7 +173,8 @@ class Ilo(object):
                 data += d
                 if not d:
                     break
-        except socket.sslerror, e: # Connection closed
+        except socket.sslerror: # Connection closed
+            e = sys.exc_info()[1]
             if not data:
                 raise IloError("Communication with %s:%d failed: %s" % (self.hostname, self.port, str(e)))
 
@@ -185,11 +192,13 @@ class Ilo(object):
             sock.connect((self.hostname, self.port))
         except socket.timeout:
             raise IloError("Timeout connecting to %s:%d" % (self.hostname, self.port))
-        except socket.error, e:
+        except socket.error:
+            e = sys.exc_info()[1]
             raise IloError("Error connecting to %s:%d: %s" % (self.hostname, self.port, str(e)))
         try:
             return ssl.wrap_socket(sock, ssl_version=ssl.PROTOCOL_TLSv1)
-        except socket.sslerror, e:
+        except socket.sslerror:
+            e = sys.exc_info()[1]
             raise IloError("Cannot establish ssl session with %s:%d: %s" % (self.hostname, self.port, e.message or str(e)))
 
     def _communicate(self, xml, protocol, progress=None):
@@ -251,7 +260,8 @@ class Ilo(object):
                         else:
                             progress(self._parse_message(d[:end], include_inform=True))
                             d = d[end:]
-        except socket.sslerror, e: # Connection closed
+        except socket.sslerror: # Connection closed
+            e = sys.exc_info()[1]
             if not data:
                 raise IloError("Communication with %s:%d failed: %s" % (self.hostname, self.port, str(e)))
 
@@ -297,7 +307,7 @@ class Ilo(object):
         # Bug in some ilo versions causes malformed XML
         if '<RIBCL VERSION="2.22"/>' in data:
             data = data.replace('<RIBCL VERSION="2.22"/>', '<RIBCL VERSION="2.22">')
-        if re.search(r'''= *[^"']''', data):
+        if re.search(r'''=+ *[^"'\n=]''', data):
             data = re.sub(r'''= *([^"'\n]+?) *\n''', r'="\1"', data)
         data = data.strip()
         if not data:
@@ -717,7 +727,6 @@ class Ilo(object):
         """Import a signed SSL certificate"""
         return self._control_tag('RIB_INFO', 'IMPORT_CERTIFICATE', text=certificate)
 
-    @untested
     def import_ssh_key(self, user_login, ssh_key):
         """Imports an SSH key for the specified user. The value of ssh_key
            should be the content of an id_dsa.pub file"""
@@ -725,13 +734,13 @@ class Ilo(object):
         if ' ' not in ssh_key:
             raise ValueError("Invalid SSH key")
         algo, key = ssh_key.split(' ',2)[:2]
-        if algo not in ('ssh-dss', 'ssh-rsa'):
-            raise ValueError("Invalid SSH key")
+        if algo != 'ssh-dss':
+            raise ValueError("Invalid SSH key, only DSA keys are supported")
         try:
             key.decode('base64')
         except Exception:
             raise ValueError("Invalid SSH key")
-        key = "-----BEGIN SSH KEY-----\r\n%s %s %s\r\n----END SSH KEY-----" % (algo, key, user_login)
+        key = "-----BEGIN SSH KEY-----\r\n%s\r\n%s\r\n%s\r\n-----END SSH KEY-----\r\n" % (algo, key, user_login)
         return self._control_tag('RIB_INFO', 'IMPORT_SSH_KEY', text=key)
 
     # Not sure how this would work, and I have no relevant hardware
@@ -742,7 +751,7 @@ class Ilo(object):
     @untested
     def delete_ssh_key(self, user_login):
         """Delete a users SSH key"""
-        return self._control_tag('USER_INFO', 'MOD_USER', elements=[etree.Element('DEL_USERS_SSH_KEY')])
+        return self._control_tag('USER_INFO', 'MOD_USER', attrib={'USER_LOGIN': user_login}, elements=[etree.Element('DEL_USERS_SSH_KEY')])
 
     def insert_virtual_media(self, device, image_url):
         """Insert a virtual floppy or CDROM. Note that you will also need to
